package be.iminds.ilabt.jfed.ui.javafx.probe_gui.command_arguments;

import javafx.beans.binding.Bindings;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.control.ComboBox;
import javafx.scene.control.TextField;
import javafx.util.StringConverter;
import javafx.util.converter.NumberStringConverter;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.IOException;
import java.net.URL;
import java.util.Arrays;
import java.util.List;
import java.util.ResourceBundle;
import java.util.stream.Collectors;

/**
 * StringArgumentChooser
 */
public class EnumArgumentChooser<E extends Enum> extends CommandArgumentChooser<E> implements Initializable {
    @FXML private ComboBox<String> valueField;

    @Nonnull private final Class<E> enumClass;
    @Nonnull private final ObservableList items;
    @Nonnull private final ObjectProperty enumProperty = new SimpleObjectProperty();

    public EnumArgumentChooser(@Nonnull Class<E> enumClass, @Nullable E defaultValue) {
        this.enumClass = enumClass;

        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("EnumArgumentChooser.fxml"));
        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);

        try {
            fxmlLoader.load();

            List<String> itemStrings = Arrays.asList(enumClass.getEnumConstants()).stream()
                    .map(Enum::name)
                    .collect(Collectors.toList());
            items = FXCollections.observableList(itemStrings);
            valueField.setItems(items);

            if (defaultValue != null) {
                valueField.getSelectionModel().select(defaultValue.name());
                enumProperty.setValue(defaultValue.name());
            } else {
                valueField.getSelectionModel().selectFirst();
                enumProperty.setValue(null);
            }
        } catch (IOException exception) {
            throw new RuntimeException(exception);
        }
    }

    public final StringConverter conv = new StringConverter<E>() {
        @Override
        public String toString(E o) {
            return o.name();
        }

        @Override
        public E fromString(String s) {
            for (E cur : enumClass.getEnumConstants()) {
                if (cur.name().equals(s))
                    return cur;
            }
            return null;
        }
    };

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {

        valueField.getSelectionModel().selectedItemProperty().addListener(new ChangeListener<String>() {
            @Override
            public void changed(ObservableValue observableValue, String oldValue, String newValue) {
                enumProperty.setValue(conv.fromString(newValue));
            }
        });

        value = enumProperty;
    }

    public void setValue(Enum v) {
        valueField.getSelectionModel().select(v.name());
    }
}
